@@ -6,6 +6,11 @@ from chameleon.core import utils
 
 marker = ()
 
+def translate_expression(value, mapping=None, default=None):
+    format = "%%(translate)s(%s, domain=%%(domain)s, mapping=%s, " \
+             "target_language=%%(language)s, default=%s)"
+    return types.template(format % (value, mapping, default))
+
 class Assign(object):
     """
     >>> from chameleon.core import testing
@@ -109,8 +114,9 @@ class Assign(object):
         symbols = stream.symbols.as_dict()
         variable = variable % symbols
 
-        if value.symbol_mapping:
-            stream.symbol_mapping.update(value.symbol_mapping)
+        if isinstance(value, types.expression):
+            if value.symbol_mapping:
+                stream.symbol_mapping.update(value.symbol_mapping)
 
         if isinstance(value, types.template):
             value = types.value(value % symbols)
@@ -152,9 +158,11 @@ class Assign(object):
 
             for i in range(_v_count):
                 stream.restore()
+        elif isinstance(value, basestring):
+            stream.write("%s = %r" % (variable, value))
         else:
             raise TypeError("Can't assign value of type %s" % type(value))
-        
+
     def end(self, stream):
         pass
 
@@ -669,17 +677,27 @@ class Tag(object):
                     temp, repr(default_string)))
                 stream.outdent()
 
-                stream.write("if %s is None or %s is False:" % (temp, temp))
+                # ignore trivial values
+                stream.write("if %s is not None and %s is not False:" % (
+                    temp, temp))
                 stream.indent()
-                stream.write("pass")
+
+                # translate non-basic values
+                stream.write(
+                    "if %s.__class__ not in (str, unicode, int, float):" % temp)
+                stream.indent()
+                stream.write("%s = unicode(%s)" % (
+                    temp, translate_expression(temp) % (
+                    stream.symbols.as_dict())))
                 stream.outdent()
-                stream.write("else:")
-                stream.indent()
 
                 # if an encoding is specified, we need to check
                 # whether we're dealing with unicode strings or not,
                 # before writing out the attribute
+                stream.write("else:")
+                stream.indent()
                 stream.ensure_unicode(temp)
+                stream.outdent()
 
                 # escape expression
                 stream.escape(temp, escape_double_quote=True)
@@ -719,10 +737,24 @@ class Tag(object):
     def end(self, stream):
         if self.cdata:
             stream.out(']]>'); return
-            
+
         if not self.selfclosing:
             stream.out('</%s>' % self.tag)
 
+
+class Attrs(object):
+    def __init__(self, defaults, name):
+        self.defaults = defaults
+        self.name = name
+
+    def begin(self, stream):
+        stream.symbol_mapping[self.name] = self.defaults
+        stream.write("attrs = %s" % self.name)
+
+    def end(self, stream):
+        pass
+
+
 class Repeat(object):
     """
     >>> from chameleon.core import testing
@@ -742,11 +774,11 @@ class Repeat(object):
     ...     "print (i, r.index, r.letter(), r.Letter(), r.roman(), r.Roman(), r.start, r.end, r.number(), r.odd(), r.even())")
     >>> _repeat.end(stream)
     >>> exec stream.getvalue()
-    (0, 0, 'a', 'A', 'i', 'I', True, False, 1, False, True)
-    (1, 1, 'b', 'B', 'ii', 'II', False, False, 2, True, False)
-    (2, 2, 'c', 'C', 'iii', 'III', False, False, 3, False, True)
-    (3, 3, 'd', 'D', 'iv', 'IV', False, False, 4, True, False)
-    (4, 4, 'e', 'E', 'v', 'V', False, True, 5, False, True)
+    (0, 0, 'a', 'A', 'i', 'I', True, False, 1, '', 'even')
+    (1, 1, 'b', 'B', 'ii', 'II', False, False, 2, 'odd', '')
+    (2, 2, 'c', 'C', 'iii', 'III', False, False, 3, '', 'even')
+    (3, 3, 'd', 'D', 'iv', 'IV', False, False, 4, 'odd', '')
+    (4, 4, 'e', 'E', 'v', 'V', False, True, 5, '', 'even')
     >>> _repeat.end(stream)
 
     A repeat over an empty set.
@@ -793,9 +825,9 @@ class Repeat(object):
     ...     "print (i, r.index, r.start, r.end, r.number(), r.odd(), r.even())")
     >>> _repeat.end(stream)
     >>> exec stream.getvalue()
-    (1, 0, True, False, 1, False, True)
-    (2, 1, False, False, 2, True, False)
-    (3, 2, False, True, 3, False, True)
+    (1, 0, True, False, 1, '', 'even')
+    (2, 1, False, False, 2, 'odd', '')
+    (3, 2, False, True, 3, '', 'even')
     >>> _repeat.end(stream)
 
     A repeat over a non-iterable raises an exception.
@@ -856,7 +888,7 @@ class Repeat(object):
         stream.write("%s = %s" % (
             ", ".join(self.declaration),
             ", ".join((repr(None),)*len(self.declaration))))
-        
+
         if self.repeatdict:
             variable = self.declaration[0]
             assert ',' not in variable
@@ -957,15 +989,15 @@ class Write(object):
         self.assign = Assign(value)
         self.structure = not isinstance(value, types.escape)
         self.defer = defer
-        
+
     def begin(self, stream):
         if not self.defer:
             self.write(stream)
-            
+
     def end(self, stream):
         if self.defer:
             self.write(stream)
-    
+
     def write(self, stream):
         temp = stream.save()
         symbols = stream.symbols.as_dict()
@@ -977,23 +1009,44 @@ class Write(object):
         expr = temp
 
         stream.write("%s = %s" % (stream.symbols.tmp, expr))
-        write("if %(tmp)s.__class__ not in (str, unicode, int, float) and hasattr(%(tmp)s, '__html__'):")
+        write("if %(tmp)s.__class__ not in (str, unicode, int, float):")
         stream.indent()
-        stream.out(types.value("%s.__html__()" % symbols['tmp']))
+        write("try:")
+        stream.indent()
+        write("%(tmp)s = %(tmp)s.__html__")
         stream.outdent()
-        
-        write("elif %(tmp)s is not None:")
+        write("except:")
+        stream.indent()
+        write("%%(tmp)s = %s" % translate_expression("%(tmp)s"))
+        stream.outdent()
+        write("else:")
+        stream.indent()
+        write("%(tmp)s = %(tmp)s()")
+        self._maybe_validate(stream)
+        stream.out(types.value(stream.symbols.tmp))
+        write("%(tmp)s = None")
+        stream.outdent()
+        stream.outdent()
+
+        write("if %(tmp)s is not None:")
         stream.indent()
 
+        # only output unicode strings
         stream.ensure_unicode(stream.symbols.tmp)
 
-        # escape non-structural values
+        # escape markup
         if not self.structure:
             stream.escape(stream.symbols.tmp)
 
+        self._maybe_validate(stream)
         stream.out(types.value(symbols['tmp']))
-        stream.outdent()
 
+        stream.outdent()
+        stream.restore()
+
+        self.assign.end(stream)
+
+    def _maybe_validate(self, stream):
         # validate XML if enabled
         if config.VALIDATION:
             try:
@@ -1003,10 +1056,7 @@ class Write(object):
                     "ElementTree (required when XML validation is enabled).")
 
             stream.symbol_mapping[stream.symbols.validate] = utils.validate
-            write("%(validate)s(%(tmp)s)")
-
-        self.assign.end(stream)
-        stream.restore()
+            stream.write("%(validate)s(%(tmp)s)" % stream.symbols.as_dict())
 
 class UnicodeWrite(Write):
     """
@@ -1087,13 +1137,13 @@ class Callback(object):
         self.visitor = visitor
         self.args = args
         self.newline = newline
-        
+
     def begin(self, stream):
-        stream.write("def %s(%s, %s, **_ignored):" % (
+        stream.write("def %s(%s, _repeat, %s, **_ignored):" % (
             self.name, stream.symbols.scope, self.args))
         stream.indent()
-        stream.write("pass")
-        
+        stream.write("if _repeat: %s.update(_repeat)" % stream.symbols.repeat)
+
         # visit slot node
         self.visitor.begin(stream)
         self.visitor.end(stream)
@@ -1145,8 +1195,8 @@ class Macro(object):
             stream.write("for _name in %s:" % stream.symbols.slots)
             stream.indent()
             if self.extend_except:
-                stream.write("if _name not in %s and _name not in (%s,):" % (
-                    stream.symbols.tmp, ", ".join(map(repr, self.extend_except))))
+                stream.write("if _name not in (%s,):" % (
+                    ", ".join(map(repr, self.extend_except))))
             else:
                 stream.write("if _name not in %s:" % stream.symbols.tmp)
 
