@@ -25,7 +25,7 @@ def parse(body, parser, validator):
     while tree is None:
         expat = xml.parsers.expat.ParserCreate()
         expatparser = ExpatParser(parser, body, expat, validator)
-        
+
         try:
             # attempt to parse this body; if we're not successful,
             # this may be because the document source consists of
@@ -51,7 +51,7 @@ def parse(body, parser, validator):
                 tree = root.getroottree()
             else:
                 raise RuntimeError("Unexpected parsing error.")
-            
+
         except xml.parsers.expat.ExpatError, e:
             offset += expat.CurrentByteIndex
 
@@ -62,13 +62,34 @@ def parse(body, parser, validator):
             code = getattr(e, 'code', -1)
             if code == 3:
                 body = None
-                
+
                 if parts:
                     continue
-                
+
                 raise
-            
-            if expatparser.root is None:
+
+            if code == 2 and expatparser.root is None:
+                body = "<meta:wrapper>%s</meta:wrapper>" % body
+                continue
+
+            if code == 9 and expatparser.root is not None:
+                # add the root as a tree fragment and update the body
+                # source to the next possible chunk
+                parts.append(expatparser.root)
+                body = body[:expatparser.index] + body[expat.CurrentByteIndex:]
+
+                # a simple heuristic is used here to allow chunks of
+                # 'junk' in-between the tree fragments
+                m = re_tag_start.search(body)
+                if m is not None:
+                    pos = m.start()
+                else:
+                    pos = -1
+                junk = body[:pos]
+                body = body[pos:]
+                parts.append(junk)
+                offset += pos
+            else:
                 # the default exception won't provide correct
                 # information about where in the document the
                 # exception stems from; we can compute it manually
@@ -80,23 +101,6 @@ def parse(body, parser, validator):
                     error_msg, line, column)
                 raise xml.parsers.expat.ExpatError(error_msg)
 
-            # add the root as a tree fragment and update the body
-            # source to the next possible chunk
-            parts.append(expatparser.root)
-            body = body[:expatparser.index] + body[expat.CurrentByteIndex:]
-            
-            # a simple heuristic is used here to allow chunks of
-            # 'junk' in-between the tree fragments
-            m = re_tag_start.search(body)
-            if m is not None:
-                pos = m.start()
-            else:
-                pos = -1
-            junk = body[:pos]
-            body = body[pos:]
-            parts.append(junk)
-            offset += pos
-            
     return tree, expatparser.doctype
 
 class ExpatParser(object):
@@ -174,7 +178,7 @@ class ExpatParser(object):
                 prefix, name = key.split(':')
                 nsmap[name] = value
                 del attrs[key]
-            
+
         for key, value in attrs.items():
             try:
                 prefix, name = key.split(':')
@@ -188,9 +192,10 @@ class ExpatParser(object):
                 if self.root is not None:
                     element = self.element
 
-                    while element is None:
+                    while element is not None:
                         namespace = element.nsmap.get(prefix)
                         if namespace is not None:
+                            nsmap[prefix] = namespace
                             break
                         element = element.getparent()
 
@@ -213,6 +218,8 @@ class ExpatParser(object):
 
         # create element using parser
         element = self.parser.makeelement(tag, attrs, nsmap=nsmap)
+        element.position = (self.expat.CurrentLineNumber,
+                            self.expat.CurrentColumnNumber)
 
         if self.root is None:
             # cook doctype
@@ -385,7 +392,7 @@ class Parser(object):
     # which invokes the ``parse``-method
     def __call__(self, body):
         return self.parse(body)
-    
+
 class Annotation(property):
     def __init__(self, name, default=None):
         property.__init__(self, self._get, self._set)
@@ -394,9 +401,11 @@ class Annotation(property):
 
     def _get(instance, element):
         value = element.attrib.get(instance.name)
-        if value is not None:
-            return loads(base64.decodestring(value))
-        return instance.default
+        if value is None:
+            return instance.default
+        if value:
+            value = loads(base64.decodestring(value))
+        return value
 
     def _set(instance, element, value):
         element.attrib[instance.name] = base64.encodestring(dumps(value))
@@ -405,7 +414,7 @@ class ElementBase(object):
     """ElementTree-compatible base element."""
 
     node = tree = parent = prefix = tail = text = None
-    
+
     def __init__(self, tag, attrib=None, nsmap=None):
         if attrib is None:
             attrib = {}
