@@ -3,8 +3,11 @@ try:
     from zope.i18n import translate
     from zope.i18nmessageid import Message
 except ImportError:
-    def fast_translate(msgid, *args, **kwargs):
-        return kwargs.get('default', msgid)
+    def fast_translate(msgid, domain=None, mapping=None, context=None,
+                       target_language=None, default=None):
+        if default is None:
+            return unicode(msgid)
+        return default
 else:
     def fast_translate(msgid, domain=None, mapping=None, context=None,
                        target_language=None, default=None):
@@ -12,9 +15,11 @@ else:
             return
 
         if target_language is not None:
-            return translate(
+            result = translate(
                 msgid, domain=domain, mapping=mapping, context=context,
                 target_language=target_language, default=default)
+            if result!=msgid:
+                return result
 
         if isinstance(msgid, Message):
             default = msgid.default
