@@ -5,7 +5,7 @@ try:
     from hashlib import sha1 as sha
 except ImportError:
     from sha import sha
-    
+
 try:
     import codegen
 except ImportError:
@@ -152,6 +152,25 @@ class Node(object):
 
         return result
 
+    def interpolated_attributes(self, internal):
+        for name, value in self.element.attrib.items():
+            if name in internal:
+                continue
+            if '}' in name:
+                namespace, name = name[1:].split('}')
+                if namespace not in self.ns_omit:
+                    for prefix, ns in self.element.nsmap.items():
+                        if ns == namespace:
+                            break
+                    else:
+                        continue
+                    name = "%s:%s" % (prefix, name)
+            parts = self.element.translator.split(value)
+            for part in parts:
+                if isinstance(part, types.expression):
+                    yield types.declaration((name,)), types.join(parts)
+                    break
+
     @property
     def attribute_ordering(self):
         static = self.static_attributes
@@ -200,7 +219,7 @@ class Node(object):
         if isinstance(self.skip, types.expression):
             if len(self.element):
                 condition.end(self.stream)
-                    
+
     def visit(self):
         assert self.stream is not None, "Must use ``start`` method."
 
@@ -253,7 +272,7 @@ class Node(object):
                     _.append(clauses.Write(part, defer=True))
                 else:
                     _.append(clauses.Out(part, defer=True))
-            
+
         # macro method
         macro = self.macro
         if macro is not None:
@@ -261,8 +280,19 @@ class Node(object):
                 dictionary = self.symbols.remote_scope
             else:
                 dictionary = self.symbols.scope
+
+            exclude = set((
+                self.symbols.scope, self.symbols.slots)) | \
+                self.stream.scope[0] | set(macro.args)
+
+            scope = set(itertools.chain(*self.stream.scope[1:])) | set((
+                self.symbols.out, self.symbols.write))
+
+            args = tuple(macro.args) + tuple(
+                "%s=%s" % (name, name) for name in scope if name not in exclude)
+
             _.append(clauses.Method(
-                macro.name, macro.args,
+                macro.name, args,
                 decorators=macro.decorators,
                 dictionary=dictionary))
 
@@ -280,7 +310,7 @@ class Node(object):
                     break
             else:
                 newline = False
-                
+
             if len(variables) > 1:
                 repeat = clauses.Repeat(
                     variables, expression, repeatdict=False, newline=newline)
@@ -298,7 +328,7 @@ class Node(object):
 
         content = self.content
         omit = self.omit
-        
+
         if self.define_slot:
             name = self.define_slot
             scope = set(itertools.chain(*self.stream.scope[1:]))
@@ -325,7 +355,7 @@ class Node(object):
                 (clauses.Condition(
                     types.template('isinstance(%(tmp)s, basestring)'),
                     (clauses.Slot(
-                        types.template("%(tmp)s(%(scope)s)"),
+                        types.template("%(tmp)s(%(scope)s, %(repeat)s)"),
                         scope_args),),
                     finalize=True,
                     invert=True),
@@ -354,7 +384,7 @@ class Node(object):
         # dynamic attributes
         dynamic_attrs = self.dynamic_attributes or ()
         dynamic_attr_names = []
-        
+
         for variables, expression in dynamic_attrs:
             if len(variables) != 1:
                 raise ValueError("Tuple definitions in assignment clause "
@@ -377,15 +407,15 @@ class Node(object):
 
                 if variable in attributes:
                     default = repr(attributes[variable])
-                    expression = self.translate_expression(value, default=default)
+                    expression = clauses.translate_expression(value, default=default)
                 else:
-                    expression = self.translate_expression(value)
+                    expression = clauses.translate_expression(value)
             else:
                 value = attributes.get(variable)
                 if value is not None:
                     if variable not in dynamic_attr_names:
                         value = repr(value)
-                    expression = self.translate_expression(value)
+                    expression = clauses.translate_expression(value)
                 else:
                     raise ValueError("Must be either static or dynamic "
                                      "attribute when no message id "
@@ -396,6 +426,8 @@ class Node(object):
         # tag
         text = self.text
         if omit is not True:
+            _.append(clauses.Attrs(self.static_attributes,
+                                   "_attrs_%d" % id(self.element)))
             selfclosing = not text and not dynamic and len(self.element) == 0
             tag = clauses.Tag(
                 self.tag, attributes,
@@ -424,14 +456,14 @@ class Node(object):
                         "Can't use message id with dynamic content translation.")
 
                 _.append(clauses.Assign(content, self.symbols.tmp))
-                content = self.translate_expression(
+                content = clauses.translate_expression(
                     types.value(self.symbols.tmp))
             else:
                 value = types.value(repr(utils.serialize(self.element, omit=True)))
                 _.insert(0, clauses.Assign(
                     value, "%s.value = %s" % (
                         self.symbols.default_marker_symbol, self.symbols.default)))
-                
+
             _.append(clauses.Write(content))
 
         # dynamic text
@@ -443,7 +475,7 @@ class Node(object):
 
             init_stream = types.value('_init_stream()')
             init_stream.symbol_mapping['_init_stream'] = generation.initialize_stream
-            
+
             subclauses = []
             subclauses.append(clauses.Define(
                 types.declaration((self.symbols.out, self.symbols.write)), init_stream))
@@ -458,8 +490,8 @@ class Node(object):
 
             # attempt translation
             subclauses.append(clauses.Assign(
-                self.translate_expression(
-                types.value('%(out)s.getvalue()'), 
+                clauses.translate_expression(
+                types.value('%(out)s.getvalue()'),
                 default=None), self.symbols.tmp))
 
             _.append(clauses.Group(subclauses))
@@ -575,31 +607,27 @@ class Node(object):
             # unnamed elements, the message id must be dynamic
             named_elements = [e for e in self.element
                               if e.node.translation_name]
-            
+
             unnamed_elements = [e for e in self.element
                                 if not e.node.translation_name]
 
-            if unnamed_elements and msgid:
-                elements = ()
-            else:
-                elements = tuple(self.element)
-            
-            # if no message id is provided, there are two cases:
-            #
-            # 1) all dynamic content is assigned a translation name
-            # 2) the message id needs to be generated dynamically
-            #
-            if not msgid and not unnamed_elements:
+            if not msgid and named_elements and not unnamed_elements:
                 msgid = self.create_msgid()
+                elements = named_elements
+            else:
+                elements = self.element
+
+            if msgid and not named_elements:
+                elements = ()
 
             if named_elements:
-                mapping = self.symbols.mapping
+                mapping = "%s_%d" % (self.symbols.mapping, id(self.element))
                 _.append(clauses.Assign(types.value('{}'), mapping))
             else:
                 mapping = 'None'
 
-            if not msgid:
-                text = utils.htmlescape(self.element.text.replace('%', '%%') or "")
+            if unnamed_elements or not msgid:
+                text = utils.htmlescape(self.element.text.replace('%', '%') or "")
                 _.append(clauses.Assign(types.value(repr(text)),
                                         self.symbols.msgid))
 
@@ -618,7 +646,7 @@ class Node(object):
                 # if the element is named, record it in the mapping
                 if element in named_elements:
                     name = element.node.translation_name
-                    
+
                     subclauses.append(clauses.Assign(
                         types.template('%(out)s.getvalue()'),
                         "%s['%s']" % (mapping, name)))
@@ -640,21 +668,24 @@ class Node(object):
                         types.template('%(msgid)s + %(out)s.getvalue()'),
                         self.symbols.msgid))
 
-                _.append(clauses.Group(subclauses))
+                # XXX: note that this should read:
+                # _.append(clauses.Group(subclauses))
+                #
+                # but there's a problem with multiple temporary
+                # variable assignments within the same block; this is
+                # just an easy work-around
+                _.append(clauses.Condition(
+                    types.value('True'), subclauses, finalize=True))
 
             if msgid:
                 value = types.value(repr(msgid)).replace('%', '%%')
-                if elements:
-                    default = self.symbols.marker
-                else:
-                    default = types.value(repr(
-                        utils.serialize(self.element, omit=True))).replace('%', '%%')
+                default = self.symbols.marker
             else:
                 default = types.template('%(msgid)s')
                 value = types.template("' '.join(%(msgid)s.split())")
 
             _.append(clauses.Assign(
-                self.translate_expression(
+                clauses.translate_expression(
                     value, mapping=mapping, default=default), self.symbols.result))
 
             # write translation to output if successful, otherwise
@@ -662,7 +693,7 @@ class Node(object):
             result = types.value(self.symbols.result)
             result.symbol_mapping[self.symbols.marker] = i18n.marker
 
-            if msgid and elements:
+            if msgid:
                 condition = types.template('%(result)s is not %(marker)s')
                 _.append(clauses.Condition(
                     condition, [clauses.UnicodeWrite(result)], finalize=True))
@@ -671,26 +702,27 @@ class Node(object):
                 if self.element.text:
                     subclauses.append(clauses.Out(
                         utils.htmlescape(self.element.text)))
+
                 for element in self.element:
                     name = element.node.translation_name
                     if name:
                         value = types.value("%s['%s']" % (mapping, name))
-                        subclauses.append(clauses.Write(value))
-                    else:
-                        subclauses.append(clauses.Out(utils.serialize(element)))
+                        subclauses.append(clauses.UnicodeWrite(value))
 
-                    for part in reversed(element.node.tail):
-                        if isinstance(part, types.expression):
-                            subclauses.append(clauses.Write(part))
-                        else:
-                            subclauses.append(clauses.Out(
-                                utils.htmlescape(part)))
+                        for part in reversed(element.node.tail):
+                            if isinstance(part, types.expression):
+                                subclauses.append(clauses.Write(part))
+                            else:
+                                subclauses.append(clauses.Out(
+                                    utils.htmlescape(part)))
+                    else:
+                        subclauses.append(clauses.Visit(element.node))
 
                 if subclauses:
                     _.append(clauses.Else(subclauses))
             else:
                 _.append(clauses.UnicodeWrite(result))
-                
+
         return _
 
     def wrap_literal(self, element):
@@ -714,9 +746,10 @@ class Node(object):
 
         for element in self.element:
             name = element.node.translation_name
-            assert name is not None
-
-            out.write("${%s}" % name)
+            if name is None:
+                out.write(utils.serialize(element, omit=True))
+            else:
+                out.write("${%s}" % name)
             out.write(element.tail)
 
         msgid = out.getvalue().strip()
@@ -724,11 +757,6 @@ class Node(object):
 
         return msgid
 
-    def translate_expression(self, value, mapping=None, default=None):
-        format = "%%(translate)s(%s, domain=%%(domain)s, mapping=%s, " \
-                 "target_language=%%(language)s, default=%s)"
-        return types.template(format % (value, mapping, default))
-
 class Element(etree.ElementBase):
     """Template element class.
 
@@ -751,7 +779,7 @@ class Element(etree.ElementBase):
             return self.element.meta_replace
 
     node = property(node)
-    
+
     def start(self, stream):
         self._stream = stream
         self.node.visit()
@@ -803,13 +831,13 @@ class MetaElement(Element):
     meta_omit = True
 
     meta_structure = True
-    
+
     meta_attributes = etree.Annotation('attributes')
 
     meta_replace = etree.Annotation('replace')
 
     meta_omit_default_prefix = etree.Annotation('omit-default-prefix')
-    
+
 class Compiler(object):
     """Template compiler.
 
@@ -867,7 +895,7 @@ class Compiler(object):
 
                 macros[define] = (extend is not None, names)
 
-    def __call__(self, macro, global_scope=True):
+    def __call__(self, macro, global_scope=True, debug=False):
         root = copy.deepcopy(self.tree).getroot()
 
         if not isinstance(root, Element):
@@ -922,7 +950,11 @@ class Compiler(object):
         stream.symbol_mapping['_init_default'] = generation.initialize_default
 
         # add code-generation lookup globals
-        stream.symbol_mapping.update(codegen.lookup_globals)
+        if debug:
+            lookup = codegen.lookup_attr_debug
+        else:
+            lookup = codegen.lookup_attr
+        stream.symbol_mapping['_lookup_attr'] = lookup
 
         if global_scope:
             assignments = (
@@ -956,11 +988,9 @@ class Compiler(object):
             clause.end(stream)
 
         if macro is not None:
-            if global_scope is False and macro == "" and 'xmlns' in root.attrib:
-                del root.attrib['xmlns']
-
             nsmap = {}
             namespaces = set()
+
             for tag in root.attrib:
                 if '}' not in tag:
                     continue
@@ -971,14 +1001,16 @@ class Compiler(object):
             for prefix, namespace in root.nsmap.items():
                 if namespace in namespaces:
                     nsmap[prefix] = namespace
-                
+
             wrapper = self.tree.parser.makeelement(
-                utils.meta_attr('wrapper'), root.attrib.copy(), nsmap=nsmap)
+                utils.meta_attr('wrapper'),
+                utils.get_attributes_from_namespace(root, config.META_NS),
+                nsmap=nsmap)
             wrapper.append(root)
             root = wrapper
 
         # output XML headers, if applicable
-        if not macro:
+        if global_scope is True or macro is "":
             header = ""
             if self.xml_declaration is not None:
                 header += self.xml_declaration + '\n'
