@@ -65,6 +65,38 @@ XML_PREFIX_MAX_LENGTH = max(map(len, XML_PREFIXES))
 s_counter = 0
 marker = object()
 
+class callablestr(str):
+    __slots__ = ()
+
+    def __call__(self):
+        return self
+
+class callableint(int):
+    __slots__ = ()
+
+    def __call__(self):
+        return self
+
+class descriptorstr(object):
+    __slots__ = "function", "__name__"
+
+    def __init__(self, function):
+        self.function = function
+        self.__name__ = function.__name__
+
+    def __get__(self, context, cls):
+        return callablestr(self.function(context))
+
+class descriptorint(object):
+    __slots__ = "function", "__name__"
+
+    def __init__(self, function):
+        self.function = function
+        self.__name__ = function.__name__
+
+    def __get__(self, context, cls):
+        return callableint(self.function(context))
+
 def sniff_type(text):
     """Return 'text/xml' if text appears to be XML, otherwise return None."""
     for prefix in XML_PREFIXES:
@@ -139,11 +171,12 @@ def _elements_with_attribute(element, ns, name, value):
             yield match
 
 class attribute(property):
-    def __init__(self, tokens, factory=None, default=None, encoding=None):
+    def __init__(self, tokens, factory=None, default=None, encoding=None, recursive=False):
         self.tokens = tokens
         self.factory = factory
         self.default = default
         self.encoding = encoding
+        self.recursive = recursive
         property.__init__(self, self.__call__)
 
     def __call__(attribute, element):
@@ -162,6 +195,11 @@ class attribute(property):
             f = attribute.factory(element.translator)
             return f(value)
 
+        if attribute.recursive:
+            parent = element.getparent()
+            if parent is not None:
+                return attribute(parent)
+
         if attribute.default is not None:
             return attribute.default
 
@@ -217,15 +255,15 @@ def serialize_element(element, encoding, omit):
         yield "<%s" % name
 
         # attributes
-        for key, value in element.attrib.items():
-            yield ' %s="%s"' % (key, escape('"', encoding=encoding))
+        for key, value in element.node.static_attributes.items():
+            yield ' %s="%s"' % (key, escape(value, '"', encoding=encoding))
 
         # elements with no text which have no children are self-closing.
         if element.text is None and len(element) == 0:
             yield ' />'; return
 
         yield '>'
-            
+
     if element.text is not None:
         yield htmlescape(element.text)
 
@@ -234,7 +272,7 @@ def serialize_element(element, encoding, omit):
             yield string
 
     if omit is False:
-        yield '</%s>' % name
+        yield '</%s>%s' % (name, element.tail)
 
 def class_hierarchy(cls):
     """Return an unordered sequence of all classes related to cls.
@@ -350,20 +388,53 @@ class repeatitem(object):
     def end(self):
         return self.index == self.length - 1
 
+    @descriptorint
     def number(self):
         return self.index + 1
 
+    @descriptorstr
     def odd(self):
-        return bool(self.index % 2)
+        """Returns a true value if the item index is odd.
 
+        >>> it = repeatitem(iter(("apple", "pear")), 2)
+
+        >>> it._iterator.next()
+        'apple'
+        >>> it.odd()
+        ''
+
+        >>> it._iterator.next()
+        'pear'
+        >>> it.odd()
+        'odd'
+        """
+
+        return self.index % 2 == 1 and 'odd' or ''
+
+    @descriptorstr
     def even(self):
-        return not self.odd()
+        """Returns a true value if the item index is even.
+
+        >>> it = repeatitem(iter(("apple", "pear")), 2)
+
+        >>> it._iterator.next()
+        'apple'
+        >>> it.even()
+        'even'
+
+        >>> it._iterator.next()
+        'pear'
+        >>> it.even()
+        ''
+        """
+
+        return self.index % 2 == 0 and 'even' or ''
 
     def next(self):
         raise NotImplementedError(
             "Method not implemented (can't update local variable).")
 
-    def letter(self, base=ord('a'), radix=26):
+    def _letter(self, base=ord('a'), radix=26):
         """Get the iterator position as a lower-case letter
 
         >>> it = repeatitem(iter(("apple", "pear", "orange")), 3)
@@ -380,7 +451,7 @@ class repeatitem(object):
         >>> it.letter()
         'c'
         """
-        
+
         index = self.index
         if index < 0:
             raise TypeError("No iteration position")
@@ -391,6 +462,9 @@ class repeatitem(object):
             if not index:
                 return s
 
+    letter = descriptorstr(_letter)
+
+    @descriptorstr
     def Letter(self):
         """Get the iterator position as an upper-case letter
 
@@ -409,8 +483,9 @@ class repeatitem(object):
         'C'
         """
 
-        return self.letter(base=ord('A'))
+        return self._letter(base=ord('A'))
 
+    @descriptorstr
     def Roman(self, rnvalues=(
                     (1000,'M'),(900,'CM'),(500,'D'),(400,'CD'),
                     (100,'C'),(90,'XC'),(50,'L'),(40,'XL'),
@@ -439,6 +514,7 @@ class repeatitem(object):
             s = s + r * rct
         return s
 
+    @descriptorstr
     def roman(self):
         """Get the iterator position as a lower-case roman numeral
 
@@ -629,73 +705,82 @@ def format_kwargs(kwargs):
 def raise_exc(exc):
     raise exc
 
-def raise_template_exception(description, kwargs, exc_info):
-    """Re-raise exception raised while calling ``template``, given by
-    the ``exc_info`` tuple (see ``sys.exc_info``)."""
+def raise_template_exception(kwargs=None, exc_info=None, filename=None):
+    if kwargs is not None:
+        kwargs = kwargs.copy()
 
-    kwargs = kwargs.copy()
+        # merge dynamic scope
+        kwargs.update(kwargs.pop('econtext', {}))
 
-    # merge dynamic scope
-    kwargs.update(kwargs.pop('econtext', {}))
+        # omit keyword arguments that begin with an underscore; these are
+        # used internally be the template engine and should not be exposed
+        map(kwargs.__delitem__, [k for k in kwargs if k.startswith('_')])
 
-    # omit keyword arguments that begin with an underscore; these are
-    # used internally be the template engine and should not be exposed
-    map(kwargs.__delitem__, [k for k in kwargs if k.startswith('_')])
+        # format keyword arguments; consecutive arguments are indented for
+        # readability
+        try:
+            formatted_arguments = format_kwargs(kwargs)
+        except:
+            # the ``pprint.pformat`` method calls the representation
+            # method of the arguments; this may fail and since we're
+            # already in an exception handler, there's no point in
+            # pursuing this further
+            formatted_arguments = ()
 
-    # format keyword arguments; consecutive arguments are indented for
-    # readability
-    try:
-        formatted_arguments = format_kwargs(kwargs)
-    except:
-        # the ``pprint.pformat`` method calls the representation
-        # method of the arguments; this may fail and since we're
-        # already in an exception handler, there's no point in
-        # pursuing this further
-        formatted_arguments = ()
+        for index, string in enumerate(formatted_arguments[1:]):
+            formatted_arguments[index+1] = " "*15 + string
 
-    for index, string in enumerate(formatted_arguments[1:]):
-        formatted_arguments[index+1] = " "*15 + string
+        kwargs = formatted_arguments
 
     # extract line number from traceback object
     cls, exc, tb = exc_info
-    try:
-        lineno = tb.tb_next.tb_frame.f_lineno - 1
-        filename = tb.tb_next.tb_frame.f_code.co_filename
-        source = open(filename).read()
-    except AttributeError:
-        source = None
 
-    # locate source code annotation (these are available from
-    # the template source as comments)
-    if source:
-        source = source.split('\n')
-        for i in reversed(range(lineno)):
-            try:
-                line = source[i]
-            except IndexError:
-                raise cls, exc, tb
-            m = re_annotation.match(line)
-            if m is not None:
-                annotation = m.group(1)
+    annotation = None
+    if filename is None:
+        # go to nearest template frame
+        while tb is not None:
+            filename = tb.tb_frame.f_globals.get('__filename__')
+            if filename is not None:
                 break
+            tb = tb.tb_next
         else:
-            annotation = ""
+            filename = "<string>"
 
-        error_msg = "Caught exception rendering template."
-    else:
-        annotation = ""
-        error_msg = "Caught exception processing template: %s.\n" % description
+        tb_next = tb
+        while tb_next is not None and '__filename__' in tb_next.tb_frame.f_globals:
+            tb = tb_next
+            tb_next = tb.tb_next
+
+        lineno = tb.tb_frame.f_lineno - 1
+        try:
+            source = open(tb.tb_frame.f_code.co_filename).read()
+        except IOError:
+            source = None
+
+        if source:
+            source = source.split('\n')
+            for i in reversed(range(lineno)):
+                try:
+                    line = source[i]
+                except IndexError:
+                    raise cls, exc, tb
+                m = re_annotation.match(line)
+                if m is not None:
+                    annotation = m.group(1)
+                    break
+
+    error_msg = "Caught exception rendering template."
 
     __dict__ = exc.__dict__
     __name__ = cls.__name__
-    
+
     error_string = str(exc)
 
     if issubclass(cls, Exception):
         class RuntimeError(cls):
             def __init__(self, *args, **kwargs):
                 pass
-            
+
             def __str__(self):
                 return "%s\n%s\n%s: %s" % (
                     error_msg, str(self.__debug_info__),
@@ -712,25 +797,28 @@ def raise_template_exception(description, kwargs, exc_info):
             debug_info = RuntimeError.__debug_info__
         except AttributeError:
             debug_info = RuntimeError.__debug_info__ = DebugInfo()
-        debug_info.append(annotation, description, formatted_arguments)
+
+        info = ""
+        if annotation:
+            info += " - Expression: ``%s``\n" % annotation
+        if filename:
+            info += " - Filename:   %s\n" % filename
+        if kwargs:
+            info += " - Arguments:  %s\n" % "\n".join(kwargs)
+
+        debug_info.append(info)
 
     raise cls, exc, tb
 
 class DebugInfo(object):
     def __init__(self):
         self.info = []
-        
-    def append(self, *args):
-        self.info.append(args)
+
+    def append(self, text):
+        self.info.append(text)
 
     def __str__(self):
-        error_format = ' - Expression: "%s"\n' \
-                       ' - Instance:   %s\n' \
-                       ' - Arguments:  %s\n'
-
-        annotation, description, formatted_arguments = self.info[-1]
-        return error_format % (
-            annotation, description, "\n".join(formatted_arguments))
+        return self.info[-1]
 
 class ValidationError(ValueError):
     def __str__(self):
